local addon = LibStub("LibzzAddOnInit"):Init(...)

local mapframe
local playerCoords
local OFFSET_X = 0.0022
local OFFSET_Y = -0.0262
local zoneText,subZoneText
local coordsline,col
local L
local outbound = true
local mf, mfn

function addon:ADDON_LOADED()
  L = self['L']
  addon:set("Debug",false)
  addon:mainFrame()
  BINDING_HEADER_COORDS = addon:GetName()
  BINDING_NAME_COORDS_TOGGLE = L["Toggle Minimap"]
end

function addon:PLAYER_LOGIN()
	addon:AddConfigEntry("toggle","hideLDBCoords",L['Hide LDB coords'],L['This hides the coords-display in your broker display'],2)
	addon:AddConfigEntry("toggle","colorLDB",L['Colorized Zone'],L['Show PvP informations about current area'],1)
  addon:AddConfigEntry("toggle","hideMapCoords",L['Hide Map coords'],L['This hides the coords-display at the bottom of your map'],2)
  addon:AddConfigEntry("toggle","ShowZone",L['Show Zone'],nil,1)
  addon:AddConfigEntry("toggle","ShowSubZone",L['Show Subzone'],nil,1)
	addon:AddConfigEntry("range","maxLength",L['Max Length'],L['Limits the maximum length of the zone'],1,0,255,1,false)
	addon:AddConfigEntry("header",'screenXY',L['Mapcoords Screen'],nil,3)
	addon:AddConfigEntry("header",'windowXY',L['Mapcoords Window'],nil,6)
	addon:AddConfigEntry("range","screenX",L['Screen X'],L['Adjusts the X-Offset on fullscreenmap'],4,-1024,1024,1,false)
	addon:AddConfigEntry("range","screenY",L['Screen Y'],L['Adjusts the Y-Offset on fullscreenmap'],5,-1024,1024,1,false)
	addon:AddConfigEntry("range","windowX",L['Window X'],L['Adjusts the X-Offset on windowed map'],7,-512,512,1,false)
	addon:AddConfigEntry("range","windowY",L['Window Y'],L['Adjusts the Y-Offset on windowed map'],8,-512,512,1,false)
  if(not addon['db']['profile']['hideMapCoords']) then
    zz_CoordsMapFrame:Show()
  else
    zz_CoordsMapFrame:Hide()
  end
	addon['db']['profile']['screenX'] = addon['db']['profile']['screenX'] or 0
	addon['db']['profile']['screenY'] = addon['db']['profile']['screenY'] or 11
	addon['db']['profile']['windowX'] = addon['db']['profile']['windowX'] or -10
	addon['db']['profile']['windowY'] = addon['db']['profile']['windowY'] or -3
  addon:ScheduleRepeatingTimer(addon['OnDataUpdate'], 1, self)
  addon:RegisterEvent("ZONE_CHANGED_NEW_AREA","ZONED")
  addon:RegisterEvent("ZONE_CHANGED","ZONED")
  addon:RegisterEvent("ZONE_CHANGED_INDOORS","ZONED")
  WorldMapFrame:HookScript("OnEnter", function(self) outbound = false end)
  WorldMapFrame:HookScript("OnLeave", function(self) outbound = true end)
  addon:ZONED()
end

function addon:Updated_Pref()
  if(not addon['db']['profile']['hideMapCoords']) then
    zz_CoordsMapFrame:Show()
  else
    zz_CoordsMapFrame:Hide()
  end
  addon:ZONED()
end

function addon:ZONED()
	if(not WorldMapFrame:IsShown()) then
		SetMapToCurrentZone()
	end
	zoneText = addon['db']['profile']['ShowZone'] and (GetRealZoneText() or GetZoneText()) or ""
	if(addon['db']['profile']['ShowSubZone']) then
		if(GetSubZoneText() and GetSubZoneText()~="") then
			if(addon['db']['profile']['ShowZone']) then
				zoneText = zoneText ~= GetSubZoneText() and format("%s, %s",zoneText,GetSubZoneText()) or zoneText
			else
				zoneText = GetSubZoneText()
			end
		elseif(not addon['db']['profile']['ShowZone']) then
			zoneText = GetRealZoneText() or GetZoneText()
		end
	end
	zoneText = addon:limitLength(zoneText,addon['db']['profile']['maxLength'])
	addon:OnDataUpdate()
end

function addon:limitLength(info, maxlength)
	if(info and maxlength) then
		if(maxlength > 0 and info:len() > maxlength) then
			info = info:sub(1,maxlength).."~"
		end
	end
	return info
end

function addon:OnButton(button)
  if(button == 'RightButton') then
    InterfaceOptionsFrame_OpenToCategory(self:GetName())
  else
    if(IsShiftKeyDown()) then
      zzCoords_MinimapToggle()
    else
      ToggleFrame(WorldMapFrame)
    end
  end
end

function zzCoords_MinimapToggle()
    if(MinimapCluster:IsVisible()) then
      MinimapCluster:Hide()
    else
      MinimapCluster:Show()
    end
end

function addon:OnTooltip()
  GameTooltip:ClearLines()
  GameTooltip:AddLine(self:get('title'))
  GameTooltip:AddLine(GetRealZoneText() or GetZoneText())
  GameTooltip:AddLine(GetSubZoneText())
  GameTooltip:AddLine(format("%s: %s",self:get('title'),tostring(playerCoords)))
  GameTooltip:AddLine(L['HINT'])
  GameTooltip:AddLine(L['HINT3'])
  GameTooltip:AddLine(L['HINT2'])
end

local function pvpColor(txt)
	local pvpType, isFFA, faction = GetZonePVPInfo()
	if(pvpType == "friendly") then
		col = "00ff00"
	elseif(pvpType == "hostile") then
		col = "ff0000"
	elseif(pvpType == "arena") then
		col = "ffff00"
	elseif(pvpType == "hostile" or pvpType == "combat" or pvpType=="contested") then
		if(faction == addon['AddOnTable']['faction']) then
			col = "886600"
		else
			col = "ff0000"
		end
	elseif(pvpType == "sanctuary") then
		col = "9999ff"
	else
		return txt
	end
	return addon:colorize(txt, col)
end

function addon:OnDataUpdate()
  addon:setPlayerCoords()
  if(zoneText and zoneText~="") then
		if(addon['db']['profile']['hideLDBCoords']) then
			coordsline = "%s"
		else
			coordsline = "%s (%s)"
		end
		addon:OnText(format(coordsline,addon['db']['profile']['colorLDB'] and pvpColor(zoneText) or zoneText,playerCoords))
  else
		if(addon['db']['profile']['hideLDBCoords']) then
			addon:OnText("")
		else
			addon:OnText(playerCoords)
		end
  end
  if(GameTooltip:GetOwner() and GameTooltip:GetOwner():GetName() and GameTooltip:GetOwner():GetName():sub(self:GetName():len() * -1) == self:GetName()) then
    addon:OnTooltip()
  end
end

function addon:round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function isoutbound()
	mf = GetMouseFocus()
	if(mf) then
		mfn = GetMouseFocus():GetName()
		if(mfn and mfn ~= "WorldMapFrame") then
			if(string.sub(mfn,1,8) == "WorldMap") then return false end
		end
	end
	return true
end

function addon:MapFrame_OnUpdate()
  local x, y = GetCursorPosition();
  x = x / WorldMapButton:GetEffectiveScale();
  y = y / WorldMapButton:GetEffectiveScale();

  local centerX, centerY = WorldMapButton:GetCenter();
  local width = WorldMapButton:GetWidth();
  local height = WorldMapButton:GetHeight();
  local adjustedY = addon:round((centerY + (height/2) - y) / height * 100,1);
  local adjustedX = addon:round((x - (centerX - (width/2))) / width * 100,1);
  zz_CoordsMapFrameCoordText:SetText(format("%s%s",isoutbound() and L['outbound'] or L['COORDFMTSTRING']( self,adjustedX, adjustedY), playerCoords))
	addon:positionFrame()
end

function addon:setPlayerCoords()
  local x,y = GetPlayerMapPosition("player")
  playerCoords = format("%.1f, %.1f", x*100, y*100)
end

--
--  Main-Frame
--

function addon:positionFrame()
  if(WORLDMAP_SETTINGS['size'] ~= WORLDMAP_WINDOWED_SIZE) then
		zz_CoordsMapFrameCoordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", addon['db']['profile']['screenX'], addon['db']['profile']['screenY'])
  else
    zz_CoordsMapFrameCoordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", addon['db']['profile']['windowX'], addon['db']['profile']['windowY'])
  end
end

function addon:mainFrame()
  mapFrame = CreateFrame("Frame", "zz_CoordsMapFrame", WorldMapFrame)
	mapFrame:SetScript("OnUpdate", function() addon:MapFrame_OnUpdate() end)

	local coordText = mapFrame:CreateFontString("zz_CoordsMapFrameCoordText", "ARTWORK")
	coordText:SetFont(GameFontNormal:GetFont())
	coordText:SetTextColor(GameFontNormal:GetTextColor())
	coordText:SetShadowColor(GameFontNormal:GetShadowColor())
	coordText:SetShadowOffset(GameFontNormal:GetShadowOffset())
	coordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", 0, 10)
end
